-- ============================================================
--
--   Ship Object Class
--
-- ============================================================


-- ============================================================
--   Class definition
-- ============================================================
--
local clsShip = class("clsShip", clsSpaceObject)
--
-- ============================================================


-- ============================================================
--   initialize : called at new()
-- ============================================================
--
function clsShip:initialize(
	objGame,
	tabShipdefinition,					-- Ship definition
	numX, numY, numHeading,				-- Initial position (px, px, rad)
	numSpeedX, numSpeedY, numRotation	-- Initial speed (px/s, px/s, rad/s)
)
	-- Check parameters and enforce default values
	if objGame == nil then
		return
	end
	if tabShipdefinition == nil then
		return
	end
	if numX == nil then
		numX = objGame.options.screenWidth/2
	end
	if numY == nil then
		numY = objGame.options.screenHeight/2
	end
	if numHeading == nil then
		numHeading = math.rad(-90)
	end
	if numSpeedX == nil then
		numSpeedX = 0
	end
	if numSpeedY == nil then
		numSpeedY = 0
	end
	if numRotation == nil then
		numRotation = 0
	end

	-- Translate ship definition and store them in local variables
	local strClassName        = tabShipdefinition.name
	local sprSprite           = tabShipdefinition.sprite
	local numNX               = tabShipdefinition.nX
	local numNX               = tabShipdefinition.nY
	local numAccelerationMax  =           20+(tabShipdefinition.stats.acceleration/20*100)
	local numDecelerationMax  =           20+(tabShipdefinition.stats.deceleration/20*100)
	local numRotationSpeedMax = math.rad( 20+(tabShipdefinition.stats.turnRate    /20*180))
	local numMiningRange      =          100+(tabShipdefinition.stats.miningBeam  /20*400)

	-- Call parent class
	clsSpaceObject.initialize(self, objGame, sprSprite, numNX, numNY, 1, numX, numY, numHeading, numSpeedX, numSpeedY, numRotation)

	-- Unset New flag
	self.isNew = 0

	-- Capacity properties
	self.capabilities = {
		acceleration  =  numAccelerationMax, -- Pixel/sec/sec
		deceleration  =  numDecelerationMax, -- Pixel/sec/sec
		rotationSpeed =  numRotationSpeedMax -- Rad/sec
	}

	-- DEBUG
	-- print ( "objShip.capabilities" )
	-- for k, v in pairs(self.capabilities) do
	-- 	print ( "\t", k, v )
	-- end
	-- print ( "" )

	-- Acceleration vector
	self.acceleration = {x = 0, y = 0}
end
--
-- ============================================================


-- ============================================================
--   draw : called by the game's global love.draw()
-- ============================================================
--
function clsShip:draw()
	-- Draw the base object
	clsSpaceObject.draw(self)

	-- Draw heading and speed in green
	love.graphics.setColor(0,255,0)
	clsSpaceObject.drawHeadingMarker(self)
	clsSpaceObject.drawSpeedVector(self)
	love.graphics.setColor(255,255,255)
end
--
-- ============================================================


-- ============================================================
--   update : called by the game's global love.update()
-- ============================================================
--
function clsShip:update(dt, cmdRotation, cmdAcceleration)
	-- Initial
	self.acceleration.x, self.acceleration.y = 0, 0

	-- Update rotation speed
	self.speed.rotation = cmdRotation*self.capabilities.rotationSpeed

	-- Convert acceleration command to actual acceleration towards the ship's heading
	if cmdAcceleration==1 then
		self.acceleration.x = cmdAcceleration*self.capabilities.acceleration*math.cos(self.position.heading)
		self.acceleration.y = cmdAcceleration*self.capabilities.acceleration*math.sin(self.position.heading)
	elseif cmdAcceleration==-1 then
		self.acceleration.x = cmdAcceleration*self.capabilities.deceleration*math.cos(self.position.heading)
		self.acceleration.y = cmdAcceleration*self.capabilities.deceleration*math.sin(self.position.heading)
	end

	-- DEBUG
	-- if self.acceleration.x~=0 or self.acceleration.y~=0 then
	-- 	print ( cmdRotation, cmdAcceleration )
	-- 	print ( self.acceleration.x, self.acceleration.y )
	-- 	print ( "" )
	-- end

	-- Update speed in pixel/sec from acceleration
	self.speed.x = self.speed.x + dt*self.acceleration.x
	self.speed.y = self.speed.y + dt*self.acceleration.y

	-- Finally call parent class
	clsSpaceObject.update(self, dt)
end
--
-- ============================================================


-- ============================================================
--   Return the class object
-- ============================================================
--
return clsShip
--
-- ============================================================
