-- ============================================================
--
--   All Space Object Class
--
-- ============================================================


-- ============================================================
--   Class definition
-- ============================================================
--
local clsSpaceObject = class("clsSpaceObject", clsActor)
--
-- ============================================================


-- ============================================================
--   initialize : called at new()
-- ============================================================
--
function clsSpaceObject:initialize(
	objGame,
	sprSprite, numNX, numNY, numScale,
	numX, numY, numHeading,
	numSpeedX, numSpeedY, numRotationSpeed
)
	-- Check parameters and enforce default values : sprite image is mandatory
	if objGame == nil then
		return
	end
	if sprSprite == nil then
		return
	end
	if numNX == nil then
		numNX = 6
	end
	if numNY == nil then
		numNY = 6
	end
	if numScale == nil then
		numScale = 1
	end
	if numX == nil then
		numX = objGame.options.screenWidth/2
	end
	if numY == nil then
		numY = objGame.options.screenHeight/2
	end
	if numHeading == nil then
		numHeading = 0
	end
	if numSpeedX == nil then
		numSpeedX = 0
	end
	if numSpeedY == nil then
		numSpeedY = 0
	end
	if numRotationSpeed==nil then
		numRotationSpeed = 0
	end

	-- Call parent class
	clsActor.initialize(self, objGame, numX, numY, numHeading, numSpeedX, numSpeedY, numRotationSpeed)

	-- Sprite properties
	self.sprite = {
		image  = sprSprite,
		number = {x = numNX, y = numNY},
		scale = numScale,
		frame  = {}
	}
	numImageWidth, numImageHeight = self.sprite.image:getDimensions()
	self.sprite.width  = numImageWidth/self.sprite.number.x
	self.sprite.height = numImageHeight/self.sprite.number.y
	self.sprite.total  = self.sprite.number.x * self.sprite.number.y

	-- Generate the quads to display the images in the sprite
	-- frame[i] corresponds to [i°, i+10°[
	for y=0, self.sprite.number.y-1, 1 do
		for x=0, self.sprite.number.x-1, 1 do
			i = x+y*self.sprite.number.x
			self.sprite.frame[i] = love.graphics.newQuad(
				x*self.sprite.width,
				y*self.sprite.height,
				self.sprite.width,
				self.sprite.height,
				self.sprite.image:getDimensions())
		end
	end
	self.collisionRadius = self.sprite.width
	if self.collisionRadius < self.sprite.height then
		self.collisionRadius = self.sprite.height
	end
	self.collisionRadius = self.collisionRadius*self.sprite.scale/2

	-- Flags
	self.isNew       = 50    -- # of frames to display a new object
	self.isDanger    = false -- Object is not yet dangerous for the player
	self.canBeMined  = false -- Object is an asteroid and is near enough from the player to be mined
	self.isCollision = false -- Object has not yet collided with player
	self.offScreen   = false -- Previously visible Projectile left the screen
end
--
-- ============================================================


-- ============================================================
--   update : called by the game's global love.update()
-- ============================================================
--
function clsSpaceObject:update(dt)
	-- Create locals from globals
	local screenWidth  = objGame.options.screenWidth
	local screenHeight = objGame.options.screenHeight

	-- First call parent class
	clsActor.update(self, dt)

	-- Check and flag if offscreen
	self.offScreen = (
			 self.position.x < -self.sprite.width  
		or self.position.y < -self.sprite.height 
		or self.position.x > screenWidth  + self.sprite.width  
		or self.position.y > screenHeight + self.sprite.height 
	)

	-- Décrémente New flag
	if self.isNew>0 then
		self.isNew = self.isNew-1
	end
end
--
-- ============================================================


-- ============================================================
--   draw : called by the game's global love.draw()
-- ============================================================
--
function clsSpaceObject:draw()
	if self.sprite.total == 1 then
		self.drawSpriteMonoframe(self)
	else
		self.drawSpriteMultiframe(self)
	end

	-- If the object is a danger, draw speed and circle in orange
	if self.isDanger then
		love.graphics.setColor(255,127,0)
	end

	-- If the object is colliding, draw speed and circle in red
	if self.isCollision then
		love.graphics.setColor(255,0,0)
	end

	-- Draw collision circle only if danger/colliding/mining
	if self.isDanger or self.isCollision or self.canBeMined then
		self:drawCollisionCircle()
	end

	-- Return to default drawing color
	love.graphics.setColor(255,255,255)
end
--
-- ============================================================


-- ============================================================
--   Draw sprite only for multiframe objects : select frame
-- ============================================================
--
function clsSpaceObject:drawSpriteMultiframe()
	-- Get current frame
	currentFrame = ( self.sprite.total - math.floor( math.deg(self.position.heading) / (360/self.sprite.total) ) ) % self.sprite.total

	-- Get corresponding quad for selecting sprite
	activeFrame = self.sprite.frame[currentFrame]

	-- Draw the sprite
	love.graphics.draw(
		self.sprite.image,
		activeFrame,
		self.position.x,
		self.position.y,
		0,
		self.sprite.scale,
		self.sprite.scale,
		self.sprite.width/2,
		self.sprite.height/2
	)
end
--
-- ============================================================


-- ============================================================
--   Draw sprite only for one-frame objects : rotate image
-- ============================================================
--
function clsSpaceObject:drawSpriteMonoframe()
	-- Draw the sprite
	love.graphics.draw(
		self.sprite.image,
		self.sprite.frame[0],
		self.position.x,
		self.position.y,
		self.position.heading,
		self.sprite.scale,
		self.sprite.scale,
		self.sprite.width/2,
		self.sprite.height/2
	)
end
--
-- ============================================================


-- ============================================================
--   Draw Heading vector on object
-- ============================================================
--
function clsSpaceObject:drawHeadingMarker()
	local numRadius = (self.sprite.width+self.sprite.height)/5

	-- Draw the heading
	love.graphics.line(
		self.position.x + numRadius*math.cos(self.position.heading),
		self.position.y + numRadius*math.sin(self.position.heading),
		self.position.x + numRadius/2*math.cos(self.position.heading) + numRadius/4*math.sin(self.position.heading),
		self.position.y + numRadius/2*math.sin(self.position.heading) - numRadius/4*math.cos(self.position.heading)
	)
	love.graphics.line(
		self.position.x + numRadius*math.cos(self.position.heading),
		self.position.y + numRadius*math.sin(self.position.heading),
		self.position.x + numRadius/2*math.cos(self.position.heading) - numRadius/4*math.sin(self.position.heading),
		self.position.y + numRadius/2*math.sin(self.position.heading) + numRadius/4*math.cos(self.position.heading)
	)
end
--
-- ============================================================


-- ============================================================
--   Draw Speed vector on object
-- ============================================================
--
function clsSpaceObject:drawSpeedVector()
	-- Draw the speed vector
	love.graphics.line(
		self.position.x,
		self.position.y,
		self.position.x + self.speed.x,
		self.position.y + self.speed.y
	)
end
--
-- ============================================================


-- ============================================================
--   Draw Collision Circle on object
-- ============================================================
--
function clsSpaceObject:drawCollisionCircle()
	-- Compute collision circle
	local numDiameter = self.sprite.width
	if self.sprite.width < self.sprite.height then
		numDiameter = self.sprite.height
	end
	numDiameter = numDiameter*self.sprite.scale

	-- Then draw it
	love.graphics.circle(
		"line",
		self.position.x,
		self.position.y,
		numDiameter/2
	)
end
--
-- ============================================================


-- ============================================================
-- function to compute collision between two objects
-- ============================================================
--
function clsSpaceObject:hasCollidedWith(objOtherSpaceObject)
	-- Declare local variable
	local numDistance           = 0
	local numNormalizedDistance = 0

	-- Return variables
	local boolIsDanger       = false
	local boolIsCollision    = false
	local boolcanBeMined     = false
	local numOverallDistance = 0

	-- If space object has just been created,no collision
	if self.isNew>0 or objOtherSpaceObject.isNew>0 then
		return false, false, nil
	end

	-- For danger flag, use distance in 1 sec, taking both speeds into account
	numDistance = math.sqrt(
			(objOtherSpaceObject.position.x+objOtherSpaceObject.speed.x-self.position.x-self.speed.x)^2
		+ (objOtherSpaceObject.position.y+objOtherSpaceObject.speed.y-self.position.y-self.speed.y)^2
	)
		numNormalizedDistance = numDistance / (self.collisionRadius + objOtherSpaceObject.collisionRadius)
	boolIsDanger = (numNormalizedDistance<1.2)

	-- For collision flag, use instant positions
	numDistance = math.sqrt(
			(objOtherSpaceObject.position.x-self.position.x)^2
		+ (objOtherSpaceObject.position.y-self.position.y)^2
	)
	local numNormalizedDistance = numDistance / (self.collisionRadius + objOtherSpaceObject.collisionRadius)
	boolIsCollision = (numNormalizedDistance<1)

	-- For mining, use overall distance, substracting both collision radii
	numOverallDistance = numDistance - (self.collisionRadius + objOtherSpaceObject.collisionRadius)
	boolcanBeMined = numOverallDistance<50

	return boolIsCollision, boolIsDanger, boolcanBeMined, numOverallDistance
end
--
-- ============================================================


-- ============================================================
--   Return the class object
-- ============================================================
--
return clsSpaceObject
--
-- ============================================================
