-- ============================================================
--
--   The game's "Menu" state
--
-- ============================================================


-- ============================================================
-- Define the state and its variables
--   Here will be the actual gameflow
-- ============================================================
--
-- Create the state
local stateMenu = clsGame:addState("Menu")
local tabDimensions = {}

-- Current ship displayed
local objShip = nil
currentShip = clsGame.currentShip

-- Set the layout for displaying the main menu and the prev/next buttons
local layoutLeft  = Layout( require("src/clsGame/stateMenu/layoutLeft") )
local layoutRight = Layout( require("src/clsGame/stateMenu/layoutRight") )
local layoutTop   = Layout( require("src/clsGame/stateMenu/layoutTop") )
local layoutMenu  = Layout( require("src/clsGame/stateMenu/layoutMenu") )

-- Give layouts style
layoutLeft:setStyle( require("src/gui/styleDefault") )
layoutRight:setStyle( require("src/gui/styleDefault") )
layoutTop:setStyle( require("src/gui/styleDefault") )
layoutMenu:setStyle( require("src/gui/styleDefault") )
--
-- ============================================================


-- ============================================================
-- Set an action for each Menu button
-- ============================================================
--
layoutMenu.btnStart:onPress(
	function()
		objGame:gotoState("Play")
	end
)
layoutMenu.btnOptions:onPress(
	function()
		objGame:gotoState("Options")
	end
)
layoutMenu.btnCredits:onPress(
	function()
		objGame:gotoState("Credits")
	end
)
layoutMenu.btnExit:onPress(
	function()
		print ( "" )
		love.event.push("quit")
	end
)
-- Set an action for Prev & Next buttons
layoutLeft.btnLeft:onPress(
	function()
		local tabShipData = objGame.tabShipData
		currentShip = currentShip-1
		if (currentShip==0) then
			currentShip = #tabShipData
		end
		tabDimensions = stateMenu:updateLayoutsDimensions()
		stateMenu:updateShip()
	end
)
layoutRight.btnRight:onPress(
	function()
		local tabShipData = objGame.tabShipData
		currentShip = currentShip+1
		if (currentShip==#tabShipData+1) then
			currentShip = 1
		end
		tabDimensions = stateMenu:updateLayoutsDimensions()
		stateMenu:updateShip()
	end
)
--
-- ============================================================


-- ============================================================
-- Called when entering the state
-- ============================================================
--
function stateMenu:enterState()
	-- Make a local for objGame:tabShipData
	local tabShipData = objGame.tabShipData

	-- Show mouse
	love.mouse.setVisible(true)

	-- Update the layouts if the screen has been resized
	tabDimensions = stateMenu:updateLayoutsDimensions()
	stateMenu:updateLayouts()

	-- Init ship
	currentShip = math.random(1,#tabShipData)
	stateMenu:updateShip()

	-- Finally show the buttons panel
	layoutLeft:show()
	layoutRight:show()
	layoutMenu:show()
	layoutTop:show()
end
--
-- ============================================================


-- ============================================================
-- Called when exiting the state to enter another state
-- ============================================================
--
function stateMenu:exitState()
	love.mouse.setVisible(false)
	layoutMenu:hide()
	layoutLeft:hide()
	layoutRight:hide()
	layoutTop:hide()
	objShip = nil
end
--
-- ============================================================


-- ============================================================
-- update() callback
--   called instead of the regular clsGame:update()
--   when the game is in the Menu state
-- ============================================================
--
function stateMenu:update(dt)
	objShip:update(dt,1,0)
end
--
-- ============================================================


-- ============================================================
-- draw() callback
--   called instead of the regular clsGame:draw()
--   when the game is in the Menu state
-- ============================================================
--
function stateMenu:draw()
	objShip:draw()
end
--
-- ============================================================


-- ============================================================
-- Create the ship object from currentShip index
-- ============================================================
--
function stateMenu:updateShip()
	local tabShipData        = objGame.tabShipData
	local strShipName        = tabShipData[currentShip].name
	local strShipDescription = tabShipData[currentShip].description

	objShip = clsShip:new(objGame, tabShipData[currentShip], numX, numY)

	layoutTop.headerShipName.text   = strShipName
	layoutTop.txtText.text          = strShipDescription
	layoutTop.barAcceleration.value = tabShipData[currentShip].stats.acceleration/20
	layoutTop.barDeceleration.value = tabShipData[currentShip].stats.deceleration/20
	layoutTop.barTurnRate.value     = tabShipData[currentShip].stats.turnRate    /20
	layoutTop.barMiningBeam.value   = tabShipData[currentShip].stats.miningBeam  /20
	layoutTop.barCargoHold.value    = tabShipData[currentShip].stats.cargoHold   /20
	layoutTop.barShield.value       = tabShipData[currentShip].stats.shield      /20
	layoutTop.barLaser.value        = tabShipData[currentShip].stats.laser       /20
end                                                                        
--
-- ============================================================


-- ============================================================
-- Set dimensions for the state's layouts
-- ============================================================
--
	-- Constants for dimensionning the layouts
function stateMenu:updateLayoutsDimensions()
	local tabDimensions = {}

	tabDimensions.externalMargin =  5
	tabDimensions.buttonHeight   = 50
	tabDimensions.buttonSpace    = 10
	tabDimensions.menuHeight     = tabDimensions.buttonHeight+2*tabDimensions.buttonSpace
	tabDimensions.topHeight      = 200
	tabDimensions.middleHeight   = (objGame.options.screenHeight - tabDimensions.menuHeight + tabDimensions.topHeight ) / 2 
	tabDimensions.sidesWidth     = 100
	tabDimensions.sidesHeight    = 200
	tabDimensions.topTextHeight  = 200
	tabDimensions.barsWidth      = objGame.options.screenWidth - 2*tabDimensions.externalMargin

	return tabDimensions
end
--
-- ============================================================


-- ============================================================
-- Actually resize the state's layouts
-- ============================================================
--
function stateMenu:updateLayouts()
	-- Resize main menu
	layoutMenu.root.left        = tabDimensions.externalMargin
	layoutMenu.root.top         = objGame.options.screenHeight - tabDimensions.externalMargin - tabDimensions.menuHeight
	layoutMenu.root.height      = tabDimensions.menuHeight
	layoutMenu.root.width       = tabDimensions.barsWidth
	layoutMenu.btnStart.width   = tabDimensions.buttonWidth
	layoutMenu.btnOptions.width = tabDimensions.buttonWidth
	layoutMenu.btnCredits.width = tabDimensions.buttonWidth
	layoutMenu.btnExit.width    = tabDimensions.buttonWidth

	-- Resize top menu
	layoutTop.root.left         = tabDimensions.externalMargin
	layoutTop.root.top          = tabDimensions.externalMargin
	layoutTop.root.height       = tabDimensions.topHeight
	layoutTop.root.width        = tabDimensions.barsWidth

	-- Resize left arrow
	layoutLeft.root.top         = tabDimensions.middleHeight - (tabDimensions.sidesHeight/2)
	layoutLeft.root.left        = tabDimensions.externalMargin
	layoutLeft.root.height      = tabDimensions.sidesHeight
	layoutLeft.root.width       = tabDimensions.sidesWidth

	-- Resize right arrow
	layoutRight.root.top         = tabDimensions.middleHeight - (tabDimensions.sidesHeight/2)
	layoutRight.root.left        = objGame.options.screenWidth - tabDimensions.externalMargin - tabDimensions.sidesWidth
	layoutRight.root.height      = tabDimensions.sidesHeight
	layoutRight.root.width       = tabDimensions.sidesWidth
end
--
-- ============================================================
