-- ============================================================
--
--   The game's "Play" state
--
-- ============================================================


-- ============================================================
-- Define the state and its variables
--   Here will be the actual gameflow
-- ============================================================
--
-- Create the state
local statePlay = clsGame:addState("Play")

-- List of all in-game asteroids
local tabAsteroids = {}

-- List of all in-game projectiles
local tabProjectiles = {}

-- List of all animations currently playing
local tabAnimations = {}

-- List of all mining beams currently active
local tabBeams = {}
--
-- ============================================================


-- ============================================================
-- Called when entering the state
-- ============================================================
--
function statePlay:enterState()
	-- Make locals from global fields
	local tabShipData = objGame.tabShipData

	-- Generate the ship
	objShip = clsShip:new(objGame, tabShipData[currentShip], numX, numY)

	-- Generate 10 asteroids
	for i = 1, 10 do
		tabAsteroids[i] = statePlay:generateAsteroid("asteroid-D00")
		tabAsteroids[i].isNew = 0
	end

	-- Reset time for scoring
	objGame.status.elapsedTime        = 0
	objGame.status.spawnTimer         = 0
	objGame.status.fireTimer          = 0
	objGame.status.oreMined           = 0
	objGame.status.runScore          = 0
	if objGame.status.exitHit then
		objGame.status.gameScore = 0
		objGame.status.exitHit   = false
	end
	objGame.status.asteroidsDestroyed = 0
end
--
-- ============================================================


-- ============================================================
-- Called when exiting the state to enter another state
-- ============================================================
--
function statePlay:exitState()
	local oreMultiplier     = 100.0
	local timeMultiplier    =   0.1
	local asteroidsMultiplier = 1.0
	objShip        = {}
	tabAsteroids   = {}
	tabAnimations  = {}
	tabProjectiles = {}
	objGame.status.runScore = math.floor(
			objGame.status.oreMined*oreMultiplier
		- objGame.status.elapsedTime*timeMultiplier
		- objGame.status.asteroidsDestroyed*asteroidsMultiplier
	)
	objGame.status.gameScore = objGame.status.gameScore + objGame.status.runScore
end
--
-- ============================================================


-- ============================================================
-- update() callback
--   called instead of the regular clsGame:update()
--   when the game is in the Menu state
-- ============================================================
--
function statePlay:update(dt)
	--  Makes local varibles from globals
	local screenWidth = objGame.options.screenWidth
	local screenHeight = objGame.options.screenHeight

	-- Move asteroids
	for i=1,#tabAsteroids do
		tabAsteroids[i]:update(dt)
	end

	-- Move projectiles
	for i=1,#tabProjectiles do
		tabProjectiles[i]:update(dt)
	end

	-- Interprets commands
	local cmdRotation, cmdAcceleration, cmdFire, cmdExit = statePlay:interpretCommands()

	-- DEBUG : exit with Escape key
	if cmdExit then
		objGame:gotoState("GameOver")
		return
	end

	if objShip.offScreen then
		objGame:gotoState("GameOver")
		return
	end

	-- Check asteroids status in relation with the player's ship
	local numMinDistance = (screenWidth+screenHeight)*2
	local nearestAsteroid = 0
	for i=1,#tabAsteroids do
		local numOverallDistance = 0
		tabAsteroids[i].isCollision, tabAsteroids[i].isDanger, tabAsteroids[i].canBeMined, numOverallDistance = tabAsteroids[i]:hasCollidedWith(objShip)
	
		-- Find nearest asteroid
		if numOverallDistance~=nil and numMinDistance>numOverallDistance then
			numMinDistance = numOverallDistance
			nearestAsteroid = i
		end
		
		-- If collision, then exit to GameOver state
		if tabAsteroids[i].isCollision and not tabAsteroids[i].offScreen then
			objGame.status.exitHit = true
			objGame:gotoState("GameOver")
			return
		end
	end

	-- Apply commands on ship
	objShip:update(dt, cmdRotation, cmdAcceleration)
	if cmdFire then
		-- Ship fires projectiles on "space" key, at a hard-coded rate
		local fireTimerInterval = 0.2
		if (objGame.status.fireTimer>fireTimerInterval) and cmdFire then
			tabProjectiles[#tabProjectiles+1] = statePlay:generateProjectile("laser")
			objGame.status.fireTimer = 0
		end
	end

	-- Simpler to reset and repopulate beams table on each frame
	tabBeams = {}
	for i=1,#tabAsteroids do
		if tabAsteroids[i].canBeMined then
			local oreMined = tabAsteroids[i].size^3 * 0.01 * dt
			objGame.status.oreMined = objGame.status.oreMined + oreMined
			tabAsteroids[i].remainingMass = tabAsteroids[i].remainingMass - oreMined
			tabBeams[#tabBeams+1] = statePlay:generateBeam(objShip, tabAsteroids[i])
		
			-- Check if mining destroyed the asteroid
			if tabAsteroids[i].remainingMass<(tabAsteroids[i].size^3)/2 then
				tabAsteroids[i].isAlive = false
				statePlay:createAsteroidsFromShot(tabAsteroids, tabAnimations, i, "asteroid-D00")
			end
		end
	end

	-- Check asteroids collision between themselves : destroy both asteroids involved
	-- Only mark asteroids to be destroyed, as destroying them in the middle of the loop
	-- would wreck havoc in the loop...
	for i=1,#tabAsteroids do
		for j=i+1,#tabAsteroids do
			-- Check if asteroid collided or is about to collide with the other asteroid
			if tabAsteroids[i]:hasCollidedWith(tabAsteroids[j]) then
				-- Mark both asteroids as to be destroyed
				tabAsteroids[i].isAlive = false
				tabAsteroids[j].isAlive = false
				-- Create new asteroids from the collision
				statePlay:createAsteroidsFromCollision(tabAsteroids, tabAnimations, i, j, "asteroid-D00")
			end
		end
	end

	-- Check projectiles collision with asteroids : destroy both projectile and asteroid involved
	-- Only mark asteroids and projectiles to be destroyed, as destroying them in the middle of the loop
	-- would wreck havoc in the loop...
	for i=1,#tabAsteroids do
		for j=1,#tabProjectiles do
			-- Check if asteroid collided or is about to collide with the projectile
			if tabAsteroids[i]:hasCollidedWith(tabProjectiles[j]) then
				-- Mark both asteroid and projectile as to be destroyed
				tabAsteroids[i].isAlive   = false
				tabProjectiles[j].isAlive = false
				-- Increment counter
				objGame.status.asteroidsDestroyed = objGame.status.asteroidsDestroyed + 1
				-- Create new asteroids from the collision
				statePlay:createAsteroidsFromShot(tabAsteroids, tabAnimations, i, "asteroid-D00")
			end
		end
	end

	-- Mark for deletion asteroids out of the screen and whose speed make them move away from the screen
	for i=1,#tabAsteroids do
		local objAsteroid = tabAsteroids[i]
		if objAsteroid.offScreen then
			if (objAsteroid.position.x * objAsteroid.speed.x > 0) or (objAsteroid.position.y * objAsteroid.speed.y>0) then
				objAsteroid.isAlive = false
			end
		end
	end

	-- Mark for deletion asteroids out of the screen and whose speed make them move away from the screen
	for i=1,#tabProjectiles do
		local objProjectile = tabProjectiles[i]
		if objProjectile.offScreen then
			if (objProjectile.position.x * objProjectile.speed.x > 0) or (objProjectile.position.y * objProjectile.speed.y>0) then
				objProjectile.isAlive = false
			end
		end
	end

	-- Actually destroy marked asteroids
	for k=#tabAsteroids,1,-1 do
		if not tabAsteroids[k].isAlive then
			statePlay:destroyAsteroid(tabAsteroids,k)
		end
	end

	-- Actually destroy marked projectiles
	for k=#tabProjectiles,1,-1 do
		if not tabProjectiles[k].isAlive then
			statePlay:destroyProjectile(tabProjectiles,k)
		end
	end

	-- Actually destroy finished animations
	for k=#tabAnimations,1,-1 do
		if tabAnimations[k].currentFrame==-1 then
			statePlay:destroyAnimation(tabAnimations,k)
		end
	end

	-- Generate an extra asteroid outside the screen if timer expires
	local spawnTimerInterval = 1
	if objGame.status.spawnTimer>spawnTimerInterval or #tabAsteroids<5 then
		tabAsteroids[#tabAsteroids+1] = statePlay:generateAsteroid("asteroid-D00")
		objGame.status.spawnTimer = 0
	end
end
--
-- ============================================================


-- ============================================================
-- draw() callback
--   called instead of the regular clsGame:draw()
--   when the game is in the Play state
-- ============================================================
--
function statePlay:draw()
	-- Draw the beams below
	for i=1,#tabBeams do
		tabBeams[i]:draw()
	end

	-- ... then the asteroids ...
	for i=1,#tabAsteroids do
		tabAsteroids[i]:draw()
	end

	-- ... then the animations ...
	for i=1,#tabAnimations do
		tabAnimations[i]:draw()
	end

	-- ... then the projectiles ...
	for i=1,#tabProjectiles do
		tabProjectiles[i]:draw()
	end

	-- ... then the ship above
	objShip:draw()
	objShip:drawCollisionCircle()

	-- Finally draw the HUD messages
	statePlay:drawHUD()
end
--
-- ============================================================


-- ============================================================
-- Finally draw the HUD messages
-- ============================================================
--
function statePlay:drawHUD()
	-- Local variables
	local strMessage = "Example HUD text"
	local numX, numY = 10, 10

	strMessage = "Lt/Rt : rotate"
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 20
	strMessage = "Up/Dn : accelerate"
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 20
	strMessage = "Space : fire"
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 20
	strMessage = "Mining is automatic"
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 20
	strMessage = "Exit screen : end level"
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 40
	strMessage = ("Elapsed time : %6.2f s"):format(objGame.status.elapsedTime)
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 20
	strMessage = ("Ore mined : %6.2f units"):format(objGame.status.oreMined)
	love.graphics.print(strMessage, numX, numY)
	numY = numY + 20
	strMessage = ("# of asteroids : %i"):format(#tabAsteroids)
	love.graphics.print(strMessage, numX, numY)
end
--
-- ============================================================


-- ============================================================
--   Interprets keyboard commands 
-- ============================================================
--
function statePlay:interpretCommands()
	-- Local variables
	local cmdRotation     = 0
	local cmdAcceleration = 0
	local cmdFire         = false
	local cmdExit         = false

	-- Transform keyboard arrows into movement commands
	if love.keyboard.isDown("right") then
		cmdRotation = cmdRotation + 1
	end
	if love.keyboard.isDown("left") then
		cmdRotation = cmdRotation - 1
	end
	if love.keyboard.isDown("up") then
		cmdAcceleration = cmdAcceleration + 1
	end
	if love.keyboard.isDown("down") then
		cmdAcceleration = cmdAcceleration - 1
	end

	-- Get fire command on Space key
	cmdFire = (love.keyboard.isDown("space") )

	-- DEBUG : get exit command on Escape key
	cmdExit = (love.keyboard.isDown("escape") )

	-- DEBUG
	-- print ( cmdRotation, cmdAcceleration, cmdFire, cmdExit )

	-- Return found commands
	return cmdRotation, cmdAcceleration, cmdFire, cmdExit
end
--
-- ============================================================


-- ============================================================
--   Returns a single asteroid
-- ============================================================
--
function statePlay:generateAsteroid(strImageName)
	-- Make locals from global table fields
	local tabAsteroidData = objGame.tabAsteroidData

	-- Declare locals
	local numSize, numX, numY, numHeading, numSpeedX, numSpeedY, numRotationSpeed
	local numRadiusMin, numRadiusMax
	local numRadius, numAngle

	-- Asteroid is outside the screen, but will pass through it : used to generate new asteroids during the game
	numRadiusMin = math.min( love.graphics.getWidth(), love.graphics.getHeight() ) / math.sqrt(2)
	numRadiusMax = numRadiusMin*1.5

	-- Generate position
	numRadius  = math.random(numRadiusMin, numRadiusMax)
	numAngle = math.rad(math.random()*360)
	numX     = love.graphics.getWidth() /2 + numRadius * math.cos(numAngle)
	numY     = love.graphics.getHeight()/2 + numRadius * math.sin(numAngle)

	-- Speed must be directed towards the screen
	local deltaX = numX-(love.graphics.getWidth() /2)
	local deltaY = numY-(love.graphics.getHeight()/2)
	numSpeedX  = math.random()*100*(-deltaX/math.abs(deltaX))
	numSpeedY  = math.random()*100*(-deltaY/math.abs(deltaY))

	-- Generate initial heading and rotation speed in radians
	numHeading = math.rad(math.random()*360)
	numRotationSpeed = math.rad(90-math.random()*180)

	-- Generate size
	numSize = math.random(1, 3)

	-- Actually generate the asteroid, without the isNew flag
	local sprAsteroidSprite = tabAsteroidData[1].sprite
	newAsteroid = clsAsteroid:new(objGame, sprAsteroidSprite, numSize, numX, numY, numHeading, numSpeedX, numSpeedY, numRotationSpeed)
	newAsteroid.isNew = 0
	return newAsteroid
end
--
-- ============================================================


-- ============================================================
--   Destroy a single asteroid, given by its index
-- ============================================================
--
function statePlay:destroyAsteroid(tabAsteroids, numIndexInTable)
	-- Check argument
	if numIndexInTable<1 then
		print("\t>>> ERROR while destroying asteroid : numIndexInTable ( " .. tostring(numIndexInTable) .. ")<1")
		return
	end
	if numIndexInTable>#tabAsteroids then
		print("\t>>> ERROR while destroying asteroid : numIndexInTable ( " .. tostring(numIndexInTable) .. ")>#tabAsteroids (" .. tostring(#tabAsteroids) .. ")")
		return
	end

	-- Actually destroy the asteroid
	table.remove (tabAsteroids, numIndexInTable)
end
--
-- ============================================================


-- ============================================================
--   Destroy a single animation, given by its index
-- ============================================================
--
function statePlay:destroyAnimation(tabAnimations, numIndexInTable)
	-- Check argument
	if numIndexInTable<1 then
		print("\t>>> ERROR while destroying animation : numIndexInTable ( " .. tostring(numIndexInTable) .. ")<1")
		return
	end
	if numIndexInTable>#tabAnimations then
		print("\t>>> ERROR while destroying animation : numIndexInTable ( " .. tostring(numIndexInTable) .. ")>#tabAnimations (" .. tostring(#tabnimations) .. ")")
		return
	end

	-- Actually destroy the asteroid
	table.remove (tabAnimations, numIndexInTable)
end
--
-- ============================================================


-- ============================================================
--   Returns a single projectile
-- ============================================================
--
function statePlay:generateProjectile()
	-- Make locals from global table fields
	local tabProjectilesData = objGame.tabProjectilesData

	-- Declare locals
	local numSize, numDeltaSpeed

	-- Speed can be in any direction
	numDeltaSpeed = 200

	-- Generate size
	numSize = 1

	-- Actually generate the projectile, without the isNew flag
	local sprProjectileSprite = tabProjectilesData[1].sprite
	newProjectile = clsProjectile:new(objGame, sprProjectileSprite, numSize, objShip, numDeltaSpeed)

	-- Return the newly created projectile
	return newProjectile
end
--
-- ============================================================


-- ============================================================
--   Destroy a single projectile, given by its index
-- ============================================================
--
function statePlay:destroyProjectile(tabProjectiles, numIndexInTable)
	-- Check argument
	if numIndexInTable<1 then
		print("\t>>> ERROR while destroying projectile : numIndexInTable ( " .. tostring(numIndexInTable) .. ")<1")
		return
	end
	if numIndexInTable>#tabProjectiles then
		print("\t>>> ERROR while destroying projectile : numIndexInTable ( " .. tostring(numIndexInTable) .. ")>#tabProjectiles (" .. tostring(#tabProjectiles) .. ")")
		return
	end

	-- Actually destroy the projectile
	table.remove (tabProjectiles, numIndexInTable)
end
--
-- ============================================================


-- ============================================================
--   Create new asteroids from 2 colliding asteroids
-- ============================================================
--
function statePlay:createAsteroidsFromCollision(tabAsteroids, tabAnimations, i, j, strImageName)
	-- Make locals from global table fields
	local tabAsteroidData  = objGame.tabAsteroidData
	local tabAnimationData =  objGame.tabAnimationData

	-- All generated asteroids will depart from the collision point
	-- Here, take the middle point between original asteroids
	-- Also give them the same base speed as the originals
	local numBaseX      = (tabAsteroids[i].position.x + tabAsteroids[j].position.x)/2
	local numBaseY      = (tabAsteroids[i].position.y + tabAsteroids[j].position.y)/2
	local numBaseSpeedX = (tabAsteroids[i].speed.x    + tabAsteroids[j].speed.x)/2
	local numBaseSpeedY = (tabAsteroids[i].speed.y    + tabAsteroids[j].speed.y)/2

	-- First create an animation
	local sprSprite = tabAnimationData[2].sprite
	local numNx     = tabAnimationData[2].nX
	local numNy     = tabAnimationData[2].nY
	statePlay:createAnimationFromCollision(tabAnimations, objGame, sprSprite, numNx, numNy, numBaseX, numBaseY, numBaseSpeedX, numBaseSpeedY)

		-- Different cases from sum of weights
	local numTotalWeight = tabAsteroids[i].size^3 + tabAsteroids[j].size^3

	-- Actually create the new asteroids
	statePlay:createNewAsteroidsFromCollision(tabAsteroids, objGame, sprSprite, numNx, numNy, numBaseX, numBaseY, numBaseSpeedX, numBaseSpeedY, numTotalWeight)
end
--
-- ============================================================


-- ============================================================
--   Create new asteroids from an asteroid destroyed by a shot
-- ============================================================
--
function statePlay:createAsteroidsFromShot(tabAsteroids, tabAnimations, i, strImageName)
	-- Make locals from global table fields
	local tabAsteroidData  = objGame.tabAsteroidData
	local tabAnimationData =  objGame.tabAnimationData

	-- All generated asteroids will depart from the collision point
	-- Here, take the middle point between original asteroids
	-- Also give them the same base speed as the originals
	local numBaseX      = tabAsteroids[i].position.x
	local numBaseY      = tabAsteroids[i].position.y
	local numBaseSpeedX = tabAsteroids[i].speed.x
	local numBaseSpeedY = tabAsteroids[i].speed.y

	-- Destroyed asteroid's weight
	local numTotalWeight = tabAsteroids[i].size^3

	-- First create an animation
	local sprSprite = tabAnimationData[2].sprite
	local numNx     = tabAnimationData[2].nX
	local numNy     = tabAnimationData[2].nY

	-- Actually call the animation
	statePlay:createAnimationFromCollision(tabAnimations, objGame, sprSprite, numNx, numNy, numBaseX, numBaseY, numBaseSpeedX, numBaseSpeedY)

	-- Actually create the new asteroids
	statePlay:createNewAsteroidsFromCollision(tabAsteroids, objGame, sprSprite, numNx, numNy, numBaseX, numBaseY, numBaseSpeedX, numBaseSpeedY, numTotalWeight)
end
--
-- ============================================================


-- ============================================================
--   Create an animation from a collision
-- ============================================================
--
function statePlay:createAnimationFromCollision(tabAnimations, objGame, sprSprite, numNx, numNy, numBaseX, numBaseY, numBaseSpeedX, numBaseSpeedY)
	-- Make locals from global table fields
	local tabAnimationData =  objGame.tabAnimationData

	-- First create an animation
	tabAnimations[#tabAnimations+1] = clsAnimation:new(objGame, sprSprite, numNx, numNy, 1, numBaseX, numBaseY, 0, numBaseSpeedX, numBaseSpeedY, 0)
end
--
-- ============================================================


-- ============================================================
--   Create a new asteroid
-- ============================================================
--
function statePlay:createNewAsteroidsFromCollision(tabAsteroids, objGame, sprSprite, numNx, numNy, numBaseX, numBaseY, numBaseSpeedX, numBaseSpeedY, numTotalWeight)
	-- Make locals from global table fields
	local tabAsteroidData  = objGame.tabAsteroidData

		-- Different cases from sum of weights
	local numRemainingWeight = numTotalWeight

	-- Loop while we can generate something and there are not already too many asteroids in the game
	while numRemainingWeight>1.5 and #tabAsteroids<50 do
		-- Determine the size of the asteroid from what can be generated
		-- given some mass loss in dust
		local numOldRemainingWeight = numRemainingWeight
		local numSize = 0
		if     numRemainingWeight>3^3*1.5 then
			numSize = math.random(2,3)
		elseif numRemainingWeight>2^3*1.5 then
			numSize = 2
		else
			numSize = 1
		end
		numRemainingWeight = numRemainingWeight - numSize^3*1.5

		-- Generate the new asteroid's position not too near its origin point
		local numRadius = numSize*25
		local numAngle  = math.rad(math.random(1,360))
		local numX      = numBaseX + numRadius * math.cos(numAngle)
		local numY      = numBaseY + numRadius * math.sin(numAngle)

		-- Generate speed moving away from collision position
		local numSpeed = math.random(25,50)
		local numSpeedX = numBaseSpeedX + numSpeed * math.cos(numAngle)
		local numSpeedY = numBaseSpeedY + numSpeed * math.sin(numAngle)

		-- Generate initial heading and rotation speed
		local numHeading = math.rad(math.random()*360)
		local numRotationSpeed = math.rad(90-math.random()*180)

		-- Actually generate the asteroid
		local sprAsteroidSprite = tabAsteroidData[1].sprite
		tabAsteroids[#tabAsteroids+1] = clsAsteroid:new(objGame, sprAsteroidSprite, numSize, numX, numY, numHeading, numSpeedX, numSpeedY, numRotationSpeed)
	end
end
--
-- ============================================================


-- ============================================================
--   Returns a single beam object
-- ============================================================
--
function statePlay:generateBeam(objShip, objAsteroid)
	-- Make locals from global table fields
	local tabBeamsData = objGame.tabBeamsData

	local sprBeamSprite = tabBeamsData[1].sprite
	local newBeam = clsBeam:new(objGame, sprBeamSprite, objShip, objAsteroid)
	return newBeam
end
--
-- ============================================================
